use strict;
use warnings;

use Test::More tests => 8;
use Test::Deep;

use RPS::Mail::EventReporter;
use Digest::HMAC_SHA1;

sub t01_build_packet_ipv4_only
{
	my $r = RPS::Mail::EventReporter->new({user => 'dfs',
					       password => 'dfs',
					       report_host => '10.0.0.1'});

	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('1.2.3.5', 'hand-spam');

	# Private IP addresses should be ignored
	$r->report_ip_event('192.168.2.1', 'hand-spam');
	$r->report_ip_event('10.0.0.1', 'hand-spam');
	$r->report_ip_event('127.0.0.1', 'hand-spam');
	$r->report_ip_event('172.17.1.1', 'hand-spam');

	my $packet = $r->build_packet();
	# We build a binary packet structure... unpack the bytes
	my @bytes;
	@bytes = unpack('C*', $packet);
	cmp_deeply(\@bytes,
		   [2, 3, ord('d'), ord('f'), ord('s'),
		    # 8 random bytes
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),

		    # 4 bytes of UNIX timestamp
		    ignore(), ignore(), ignore(), ignore(),

		    # Two IPv4 reports
		    1, 0, 10,

		    # The actual IP address
		    1, 2, 3, 4,

		    # greylisted event code
		    1,

		    # Another IP address
		    1, 2, 3, 5,

		    # hand-spam event code
		    4,

		    # EOR
		    0,

		    # 10 bytes of partial sha1sum
		    ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore()],
		   'Generated packet looks correct');

	# Validate the SHA1sum
	my $len = scalar(@bytes);
	my $packet_without_checksum = pack('C*', @bytes[0 .. $len-11]);
	my $ctx = Digest::HMAC_SHA1->new('dfs');
	$ctx->add($packet_without_checksum);
	my $sha1 = substr($ctx->digest, 0, 10);
	is ($sha1, pack('C*', @bytes[$len-10 .. $len-1]),
	    'HMAC_SHA1 sum is correct');
}

sub t02_build_packet_ipv6_only
{
	my $r = RPS::Mail::EventReporter->new({user => 'dfs',
					       password => 'dfs',
					       report_host => '10.0.0.1'});

	$r->report_ip_event('feff:1234:5678:9abc:def0:1234:5678:9999', 'greylisted');
	$r->report_ip_event('2002:8aed:30c5::8aed:30c5', 'valid-recip');

	# Link-local IPv6 address should be ignored
	$r->report_ip_event('fe80::2e0:18ff:feab:147f', 'valid-recip');

	my $packet = $r->build_packet();
	# We build a binary packet structure... unpack the bytes
	my @bytes;
	@bytes = unpack('C*', $packet);

	cmp_deeply(\@bytes,
		   [2, 3, ord('d'), ord('f'), ord('s'),
		    # 8 random bytes
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),

		    # 4 bytes of UNIX timestamp
		    ignore(), ignore(), ignore(), ignore(),

		    # Two IPv6 reports
		    2, 0, 34,

		    # The IPv6 address
		    0x20, 0x02, 0x8a, 0xed, 0x30, 0xc5, 0x00, 0x00,
		    0x00, 0x00, 0x00, 0x00, 0x8a, 0xed, 0x30, 0xc5,

		    # valid-recipient code
		    7,

		    # The IPv6 address
		    0xfe, 0xff, 0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc,
		    0xde, 0xf0, 0x12, 0x34, 0x56, 0x78, 0x99, 0x99,

		    # Greylist code
		    1,

		    # EOR
		    0,

		    # 10 bytes of partial sha1sum
		    ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore()],
		   'Generated packet looks correct');

	# Validate the SHA1sum
	my $len = scalar(@bytes);
	my $packet_without_checksum = pack('C*', @bytes[0 .. $len-11]);
	my $ctx = Digest::HMAC_SHA1->new('dfs');
	$ctx->add($packet_without_checksum);
	my $sha1 = substr($ctx->digest, 0, 10);
	is ($sha1, pack('C*', @bytes[$len-10 .. $len-1]),
	    'HMAC_SHA1 sum is correct');
}

sub t03_build_packet_ipv4_and_ipv6
{
	my $r = RPS::Mail::EventReporter->new({user => 'dfs',
					       password => 'dfs',
					       report_host => '10.0.0.1'});

	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('feff:1234:5678:9abc:def0:1234:5678:9999', 'greylisted');
	$r->report_ip_event('1.2.3.5', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	my $packet = $r->build_packet();
	# We build a binary packet structure... unpack the bytes
	my @bytes;
	@bytes = unpack('C*', $packet);
	cmp_deeply(\@bytes,
		   [2, 3, ord('d'), ord('f'), ord('s'),
		    # 8 random bytes
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),

		    # 4 bytes of UNIX timestamp
		    ignore(), ignore(), ignore(), ignore(),

		    # Two IPv4 reports
		    1, 0, 10,

		    # The actual IP address
		    1, 2, 3, 4,

		    # greylisted event code
		    1,

		    # Another IP address
		    1, 2, 3, 5,

		    # hand-spam event code
		    4,

		    # Two IPv6 reports
		    2, 0, 34,

		    # The IPv6 address
		    0x20, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		    0x00, 0x00, 0x00, 0x00, 0x45, 0x67, 0x89, 0xab,

		    # hand-spam code
		    4,

		    # The IPv6 address
		    0xfe, 0xff, 0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc,
		    0xde, 0xf0, 0x12, 0x34, 0x56, 0x78, 0x99, 0x99,

		    # Greylist code
		    1,

		    # EOR
		    0,

		    # 10 bytes of partial sha1sum
		    ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore()],
		   'Generated packet looks correct');

	# Validate the SHA1sum
	my $len = scalar(@bytes);
	my $packet_without_checksum = pack('C*', @bytes[0 .. $len-11]);
	my $ctx = Digest::HMAC_SHA1->new('dfs');
	$ctx->add($packet_without_checksum);
	my $sha1 = substr($ctx->digest, 0, 10);
	is ($sha1, pack('C*', @bytes[$len-10 .. $len-1]),
	    'HMAC_SHA1 sum is correct');
}

sub t04_repeated_events
{
	my $r = RPS::Mail::EventReporter->new({user => 'dfs',
					       password => 'dfs',
					       report_host => '10.0.0.1'});

	$r->set_software_and_version('WibbleMagic', '1.2');
	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('1.2.3.4', 'greylisted');
	$r->report_ip_event('feff:1234:5678:9abc:def0:1234:5678:9999', 'greylisted');
	$r->report_ip_event('1.2.3.5', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	$r->report_ip_event('2001::4567:89ab', 'hand-spam');
	my $packet = $r->build_packet();
	# We build a binary packet structure... unpack the bytes
	my @bytes;
	@bytes = unpack('C*', $packet);
	cmp_deeply(\@bytes,
		   [2, 3, ord('d'), ord('f'), ord('s'),
		    # 8 random bytes
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),

		    # 4 bytes of UNIX timestamp
		    ignore(), ignore(), ignore(), ignore(),

		    # Software name
		    6, 0, 11,
		    ord('W'), ord('i'), ord('b'), ord('b'),
		    ord('l'), ord('e'), ord('M'), ord('a'),
		    ord('g'), ord('i'), ord('c'),

		    # Software version
		    7, 0, 3,
		    ord('1'), ord('.'), ord('2'),

		    # One IPv4 report
		    1, 0, 5,

		    # The actual IP address
		    1, 2, 3, 5,

		    # hand-spam event code
		    4,

		    # One IPv6 report
		    2, 0, 17,

		    # The IPv6 address
		    0xfe, 0xff, 0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc,
		    0xde, 0xf0, 0x12, 0x34, 0x56, 0x78, 0x99, 0x99,

		    # Greylist code
		    1,

		    # One repeated IPv4 report
		    3, 0, 6,

		    # The actual IP address
		    1, 2, 3, 4,

		    # Greylisted five times
		    1, 5,

		    # One repeated IPv6 report
		    4, 0, 18,

		    # The IPv6 address
		    0x20, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		    0x00, 0x00, 0x00, 0x00, 0x45, 0x67, 0x89, 0xab,

		    # hand-spam code repeated 9 times
		    4, 9,

		    # EOR
		    0,

		    # 10 bytes of partial sha1sum
		    ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore(),
		    ignore(), ignore(), ignore(), ignore()],
		   'Generated packet looks correct');

	# Validate the SHA1sum
	my $len = scalar(@bytes);
	my $packet_without_checksum = pack('C*', @bytes[0 .. $len-11]);
	my $ctx = Digest::HMAC_SHA1->new('dfs');
	$ctx->add($packet_without_checksum);
	my $sha1 = substr($ctx->digest, 0, 10);
	is ($sha1, pack('C*', @bytes[$len-10 .. $len-1]),
	    'HMAC_SHA1 sum is correct');
}

t01_build_packet_ipv4_only();
t02_build_packet_ipv6_only();
t03_build_packet_ipv4_and_ipv6();
t04_repeated_events();
